;------------------------------------------------------------
; LCD.S (written by BLUE 2003.03.18 - 2008.05.05)
; Compatible with DOTDS routine for PB-1000/C/AI-1000/FX-870P.
; ROM[`DOTDSƓ܂B
;------------------------------------------------------------
; This is public domain program.
;------------------------------------------------------------
; [ASSEMBLE THIS FILE]
;   >HD61 lcd.s /p [enter]
; Transmit lcd.pbf --> PB-1000/AI-1000/FX-870P/VX-3.
;
;------------------------------------------------------------
; LCD Control protocol
;------------------------------------------------------------
; References :
; [Pockecom Journal] 1993/12 [DOTDS] by N.Hayashi
; [Analysis of 'LDL' instructions.] 2006/9 by Piotr Piatek
; [Analysis of "ON interrupt".] 2006/9/18 by Piotr Piatek
;  ( http://www.pisi.com.pl/piotr433/ )
; [Analysis of LCDC command.] 2006/9/24 by Miyura
; [Programming of the LCD controller] 2008/3 by Xerxes ,Piotr Piatek
;
; I hope everyone read this analysis result.
; --> http://www.pisi.com.pl/piotr433/lcd_man.zip
;
; LCD@́A|PRW[i1993/12ɍČfڂꂽ
; N.Hayashi(͂)̋LƁAPiotr Piatekɂ͌(2006/09)
; сAMiyuraɂ(2006/9)AđSĂ̏W听ƌׂ
; XerxesPiotr Piatek̉͌(2008/3)ɂĂ܂B
;
; {eLXg(LCD.S)ƋɁAPiotr̃y[WɂXerxes̉͌ʂ
; ǂނƂ߂܂B
;  http://www.pisi.com.pl/piotr433/lcd_man.zip
;
;------------------------------------------------------------
; The LCD control is possible that it does it with the next method.
; LCD́Aȉ̎菇ōsƂ\łB
; (1) Select LCD register or data ram.(use PPO operation.)
;     PPO߂ɂāALCD䃌WX^f[^RAMwB
; (2) Set control command or display data to LCD.(use STL* instruction.)
;     STL߂ɂāALCDɑ΂ĐR}h`f[^݂܂B
; (3) Reading from the LCD data port is possible by the use of 
;     the LDL* instruction. 
;     LCD̃f[^ǂݏóALDL*߂gƂŉ\ƂȂ܂B
;
; [lcd control register (8bit:write only)]
;   Bit<7> --- LCD VDD (1:ON,0:OFF)
;   Bit<6> --- LCD CLOCK (0:STOP)
;   Bit<5> --- NOT USED
;   Bit<4> --- CE4(no connect)
;   Bit<3> --- CE3(no connect)
;   Bit<2> --- CE2
;   Bit<1> --- CE1
;   Bit<0> --- OP(1:command register ,0:data ram)
;
; It is selected the data I/O ahead by executing 'PPO'. 
; for example:
;(1)select command register(R}hWX^w)
;  PPO &HDF          ;same as 'PPO &HC7'
;  PPO &HC5          ;without HD44352 ?
;  PPO &HC3          ;contrast cotrol(select HD44352?)/RgXg(HD44352w?)
;(2)select data ram(f[^RAMw)
;  PPO &HDE          ;same as 'PPO &HC6'
;  PPO &HC4          ;without HD44352 ?
;
; It is possible to do the setting of LCD with a STL operation,
; after command register is selected.
; R}hWX^wASTL߂ɂLCD̓샂[hAAhX
; ݒ\łBݒ肷Ƃ͈ȉ̏ŏo͂܂B
;
;////////////////////////////////////////
; 1. CONTROL FOR DRAW
;    LCD\
;////////////////////////////////////////
;
; [lcd command format]
;
;  [Mode],[Addres(columns)],[Address(rows)]
;
; [Mode ( 8bit )]
;   Bit<7>-<5> --- IMAGE MODE
;		   000: not Old(LCD) and New(Write data)
;		   001: XOR
;		   010: CLEAR
;		   011: DRAWC
;		   100: OVERWRITE
;		   101: OR
;		   110: CLEAR
;		   111: NO OPERATION
;   Bit<4> ------- LCD SELECT ( 0:LCD1(left part of lcd) , 1:LCD2(right part of lcd) )
;   Bit<3>-<0> --- Command for LCDC
;                  0 :unknown (Initialization?)
;                  1 :READ FROM LCD(use LDL* instruction.)
;                  2 :DRAW BITIMAGE
;                  3 :DRAW CHARACTER
;                  4 :DISPLAY ON/OFF
;                  5 :unknown
;                  6 :SPRITE DEFINITION BY GRAPHIC
;                  7 :SPRITE DEFINITION BY CHARACTER
;                  8 :SET TOP POSITION OF ROW/SET CHARACTER(SPRITE DISPLAY) WIDTH
;                  9 :SPRITE ON/OFF
;                  10:unknown (reset lcdc?)
;                  11:USER CHARACTER DEFINITION
;                  12:CHANGING CONTRAST
;                  13:CHANGING CURSOR FLASH(ON-Interrupt Frequency Control)
;                  14:SET SPRITE POSITION
;                  15:unknown
;
; [Address ( 8bit * 2 )]
; [First byte -- LCD ADDRESS(columns)]
;   Bit<7> ----- Select block(0:left part,1:right part)
;   Bit<6>-<0>-- columns(4bit addressing) 0 - 95
;
; [Second byte - LCD ADDRESS(rows)]
;   Bit<7>-<2> - NOT USED
;   Bit<1>-<0> - rows(0,1,2,3)
;
; The data output destination is set to LCD data RAM by executing 'PPO &HDE'.
; f[^݃[h́ASTL߂ɂďo͂ꂽf[^A
; ݒ肳ꂽLCDAhXo͂܂BiLQƁj
;
; for example:
;(1)display bitimage '&HAA' (rbgC[W&hAALCDɏo)
;  PPO   &HDF         ;Select command register(same as 'PPO &HC7' *1)
;  LD    $0,&H82      ;Set 'DRAW BITIMAGE','OVERWRITE', LCD1 select
;  LD    $1,8         ;Set LCD address(columns=(8/2)=4)
;  LD    $2,0         ;Set LCD address(line=0)
;  STLM  $0,3         ;Set LCD command register & address(3byte)
;  LD    $3,&HAA      ;Set data '&HAA'
;  PPO   &HDE         ;Select data ram(same as 'PPO &HC6' *1)
;  STL   $3           ;Display LCD( 8bit data )
;  RTN
;
;( note *1 )
; It is possible to do setting LCD with 'PPO &HC5/PPO &HC4'.
; In that case, the flicker of the screen decreases but the noise increases.
; (The reason is not understood well.)
; LCD̐ݒɁAPPO&HC5/PPO&HC4̃yAgƂł܂B
; ̏ꍇAʂ̂炷Ƃł܂AmCYۂȂ܂B
; (AHD44353̂ݗLɂȂ鎖ŋN錻ۂƎv܂Aڍׂ͕sł)
;
;(2)display character "A" (LN^"A"LCDɏo)
;  PPO   &HDF         ;Select command register
;  LD    $0,&H83      ;Set 'DRAW CHARACTER','OVERWRITE', LCD1 select
;  LD    $1,48        ;Set LCD address(columns=(48/16)=3)
;  LD    $2,0         ;Set LCD address(line=0)
;  STLM  $0,3         ;Set LCD command register & address(3byte)
;  LD    $3,&H14      ;Set data "A" ("A"=&H41 --> &H14)
;                                   ("2"=&H32 --> &H23)
;  PPO   &HDE         ;Select data ram
;  STL   $3           ;Display LCD( character data )
;  RTN
;
;////////////////////////////////////////
; 2. CONTROL FOR READ FROM LCD DATA PORT
;    LCDf[^|[g̓ǂݏo
;////////////////////////////////////////
; [lcd command format]
;
;  [Mode],[Addres(columns)],[Address(rows)]
;
; [Mode ( 8bit )]
;   Bit<7>-<5> --- READ MODE ( 111:DATA RAM, 101:LCDC?, 011:DATA RAM?, 001:DATA RAM? )
;   Bit<4> ------- LCD SELECT ( 0:LCD1, 1:LCD2 )
;   Bit<3>-<0> --- Command for LCDC
;                  1 :READ FROM LCD(use LDL* instruction.)
;
; [Address ( 8bit * 2 )]
; [First byte -- LCD ADDRESS(columns)]
;   Bit<7> ----- Select block(0:left part,1:right part)
;   Bit<6>-<0>-- columns(4bit addressing) 0 - 95
;
; [Second byte - LCD ADDRESS(rows)]
;   Bit<7>-<2> - NOT USED
;   Bit<1>-<0> - rows(0,1,2,3)
;
; for example:
; read bitimage from LCD1,colimn=4,rows(line)=0.(ʒu4,s0̃rbg}bv$3֓ǂݏo)
;  PPO   &HDF         ;Select command register
;  LD    $0,&HE1      ;Set 'READ FROM LCD','DATA RAM' & LCD1 select
;  LD    $1,8         ;Set LCD address(columns=(8/2)=4)
;  LD    $2,0         ;Set LCD address(line=0)
;  STLM  $0,3         ;Set LCD command register & address(3byte)
;
;  PPO   &HDE         ;Select data ram
;  LDL   $3           ;Read bitimage from LCD dataram to $3.
;                     ;(but 4bit data are swapped)
;  LD    $0,$3        ;swap 4bit $3
;  DIU   $3           ;
;  DID   $0           ;
;  OR    $3,$0        ;
;  RTN
;
;////////////////////////////////////////
; 3. CONTROL FOR CHANGING CONTRAST
;    RgXg
;////////////////////////////////////////
; [lcd command format]
;
;  [Mode/contrast0],[contrast1],[contrast2]
;
; [Mode/contrast0 ( 8bit )]
;   Bit<7>-<4> --- contrast(Bit<3>-<0>)
;   Bit<3>-<0> --- Command for LCDC
;                  12:CHANGING CONTRAST
;
; [contrast1]
;   Bit<7>-<0> --- contrast(Bit<11>-<4>)
;
; [contrast2]
;   Bit<7> ------- NOT USED
;   Bit<6>-<0> --- contrast(Bit<19>-<12>)
;
; for example:
; set contrast to the lowest value.(RgXgŒɐݒ肷)
;  LD    $0,&HC       ;Set 'CHANGING CONTRAST',contrast(Bit<3>-<0>)=0
;  LDW   $1,0         ;Set contrast1/2 data (The lowest value)
;  PPO   &HC3         ;Select command register for changing contrast.
;  STLM $0,3          ;Set contrast data to LCDC.
;  RTN
;
;////////////////////////////////////////
; 4. CONTROL FOR CHANGING CURSOR FLASH
;    J[\_Ő (PB-1000)
;////////////////////////////////////////
; [lcd command format]
;
;  [Mode]
;
; [Mode ( 8bit )]
;   Bit<7>-<4> --- MODE(Cycle of cursor blinking)
;                  0: 910kHz/4000h/2/CSRSN = 1.11Hz (default PB-1000/C)
;                  1: 910kHz/00EDh/2/CSRSN = 76.8Hz
;                  2: 910kHz/00FAh/2/CSRSN = 72.8Hz
;                  3: 910kHz/0100h/2/CSRSN = 71.1Hz
;                  4: 910kHz/0100h/2/CSRSN = 71.1Hz
;                  5: 910kHz/0100h/2/CSRSN = 71.1Hz
;                  6: 910kHz/0100h/2/CSRSN = 71.1Hz
;                  7: 910kHz/0200h/2/CSRSN = 35.5Hz
;                  8: 910kHz/0400h/2/CSRSN = 17.7Hz
;                  9: 910kHz/0800h/2/CSRSN = 8.89Hz
;                  A: 910kHz/1000h/2/CSRSN = 4.44Hz
;                  B: 910kHz/1000h/2/CSRSN = 4.44Hz
;                  C: 910kHz/2000h/2/CSRSN = 2.22Hz
;                  D: 910kHz/2000h/2/CSRSN = 2.22Hz
;                  E: 910kHz/2000h/2/CSRSN = 2.22Hz
;                  F: 910kHz/2000h/2/CSRSN = 2.22Hz
;   Bit<3>-<0> --- Command for LCDC
;                  13 :CHANGING CURSOR FLASH
;
; CSRCN(&H6951) = 25(default)
;
; ̃R}hɂALCDCMPUɓ͂Ă/ON[qMpX(HD61700 Pin11)
; 䂪ύXł܂B
; PB-1000ł́Å荞ݐM̓J[\̓_(&HFFCAs)ɂg
; Ȃ悤ŁA̐ݒύXĂJ[\̓_őxς܂񂪁A
; FX-870P/VX-4ł́AL[͊荞݂ƋpɂȂĂ炵AL[s[gx
; ύXł܂B(WL[(KEYIN)Ńs[gxύX\BMODEl=A)
; ȉAPB-1000ł̐ݒłB
;
; for example:
; The cycle of the cursor blinking is quadrupled. (J[\_Ŏ4{ɂ)
;  PPO   &HDF         ;Select command register.
;  STL   &HAD         ;Set cycle of the cursor blinking.
;  RTN
;
;////////////////////////////////////////
; 5. DISPLAY ON/OFF
;    ʕ\ON/OFF
;////////////////////////////////////////
; [lcd command format]
;
;  [Mode]
;   Bit<7>-<6> --- none (all 0)
;   Bit<4>     --- Display Control ( 1:ON, 0:OFF )
;   Bit<3>-<0> --- Command for LCDC
;                  4 :DISPLAY ON/OFF
;
; for example:
; (1)The LCD display is turned off. (ʕ\Offɂ)
;  PPO   &HDF         ;Select command register.
;  STL   &H04         ;Display off
;  RTN
;
; (2)The LCD display is turned on. (ʕ\Onɂ)
;  PPO   &HDF         ;Select command register.
;  STL   &H14         ;Display on
;  RTN
;
; ̃R}hgƁAf[^`悵ĂԂ͉ʂĂāA
; Iꊇ\悤Ȑ䂪\łB
;
;////////////////////////////////////////
; 6. SPRITE CONTROL
;    XvCg
;////////////////////////////////////////
;
; 6.0. [Tv]
;
; XvCg`āALCDɔwi(LCD data ram)Ƃ̍\\łB
; XvCgƔwi(LCD data ram)́Aʃv[ɕ\邽߁Awi\̕ύXɂ
; XvCg`悪㏑邱Ƃ͂܂B
; XvCg̕\ʒuύX/`/LCDC(n[hEFA)ɂĎs邽ߍłB
;
; 
; (1)XvCgLCD1/LCD21(v2)`ł܂B
;   LCD1()Œ`ꂽXvCǵALCD1()ł̂ݕ\\łB
;   Eɂ\ꍇ́AʓrELCDCɑ΂ăXvCg`KvłB
; (2)XvCg̑傫́A8x8sNZŒłB
;   (ʃTCYւ̒`@邩m܂񂪁A󔻂Ă܂)
; (3)XvCg̕\́A8sNZ+ItZbgʒuPʂł\ł܂B
;   Ⴆ΁AItZbg2Œ`ꍇA`łJʒúA8*2*n+2 Pʂ
;   wł܂B(rʒuŕ\悤ƂƃVtg\Ă܂܂j
;
; 6.1. [ SPRITE DEFINITION (XvCg`) ]
;
; [lcd command format]
;
;  [Mode],[horizontal offset],[dummy]
;
; [Mode( 8bit )]
;   Bit<7>-<5> --- NOT USED
;   Bit<4> ------- LCD SELECT ( 0:LCD1, 1:LCD2 )
;   Bit<3>-<0> --- Command for LCDC
;                  6 :SPRITE DEFINITION BY GRAPHIC
;                  7 :SPRITE DEFINITION BY CHARACTER
;
; [horizontal offset(ItZbgʒu)]
;   Bit<7>-<0> --- horizontal offset(0-95)
;
; [dummy]
;   Bit<7>-<0> --- NOT USED (̒lLCDCւ̐ݒ͕Kvłۂɂ͎gȂ悤ł)
;
; for example:
; (1)define the left sprite by graphic (LCDɑ΂ăXvCg`)
;  PPO   &HDF          ;Select command register.
;  LD    $0,&H06       ;Set 'SPRITE DEFINITION BY GRAPHIC' & LCD1 select
;  LD    $1,6          ;horizontal offset = 3
;  STLM  $0,3          ;Set LCD command (3byte)
;  PRE   IX,SPRITE     ;
;  LDM   $0,(IX+$31),8 ;$0 - $7 <-- graphic data
;  PPO   &HDE          ;Select data ram
;  STLM  $0,8          ;set to lcdc(8 byte)
;  RTN
; SPRITE:
;  DB &H3C,&H42,&H91,&H85 ;Graphic pattern data
;  DB &H85,&H91,&H42,&H3C ;
;
; (2)define the right sprite by character'A'(ELCDɑ΂ĕ"A"XvCg`)
;  PPO   &HDF          ;Select command register.
;  LD    $0,&H17       ;Set 'SPRITE DEFINITION BY CHARACTER' & LCD2 select
;  LD    $1,6          ;horizontal offset = 3
;  STLM  $0,3          ;Set LCD command (3byte)
;  LD    $3,&H14       ;Set data "A" ("A"=&H41 --> &H14)
;                                    ("2"=&H32 --> &H23)
;  PPO   &HDE          ;Select data ram
;  STL   $3            ;Display LCD( character data )
;  RTN
;
;
; 6.2. [ SET CHARACTER(SPRITE DISPLAY) WIDTH (yуXvCg\̐ݒ) ]
;
; [lcd command format]
;
;  [Mode]
;
; [Mode( 8bit )]
;   Bit<7>-<6> --- top position of row ( 00:row0(default) ,01:row1 ,10:row2 ,11:row3 )
;   Bit<5>-<4> --- character(sprite display) width
;                  ( 00:8pixel ,01:7pixel ,10:6pixel(default) ,11:5pixel )
;   Bit<3>-<0> --- Command for LCDC
;                  8 :SET TOP POSITION OF ROW/SET CHARACTER(SPRITE) WIDTH
;
; for example:
; define character(sprite display) width = 8picel (yуXvCg\8sNZɂ)
;  PPO   &HDF          ;Select command register.
;  STL   &H08          ;Set CHARACTER(SPRITE DISPLAY) WIDTH=8 ,TOP POSITION OF ROW=0(default)
;  RTN
;
; XvCĝ͌Œ蕝(8sNZ)Ȃ̂łAftHgƕ\LN^
; (6sNZ)ŏo͂Ă܂߁A8sNZ\sۂ́A\Oɖ{R}h
; sKv܂B
; ̃R}hrow(s)̊Jnʒu`邱ƂłAn[hEFAXN[
; sƂł܂Bڍׂ́A"8. SCROLL CONTROL" ĉB
;
;
; 6.3. [ SPRITE ON/OFFiXvCg\ ON/OFFj ]
;
; [lcd command format]
;
;  [Mode]
;
; [Mode( 8bit )]
;   Bit<7>-<5> --- SPRITE IMAGE MODE
;		   000: not Old(LCD) and New(Write data)
;		   001: XOR
;		   010: CLEAR
;		   011: DRAWC
;		   100: OVERWRITE
;		   101: OR
;		   110: CLEAR
;		   111: NO OPERATION
;   Bit<4>     --- Display Control ( 1:ON, 0:OFF )
;   Bit<3>-<0> --- Command for LCDC
;                  9 :SPRITE ON/OFF
;
; for example:
; (1)The sprite display is turned off. (XvCg\Offɂ)
;  PPO   &HDF         ;Select command register.
;  STL   &H09         ;sprite display off
;  RTN
;
; (2)The sprite display is turned on. (XvCg\Onɂ)
;  PPO   &HDF         ;Select command register.
;  STL   &H99         ;Set "OVERWRITE", sprite display on
;  RTN
;
;
; 6.4. [ SET SPRITE POSITION(XvCg\ʒuw) ]
;
; [lcd command format]
;
;  [Mode],[Addres(columns)],[Address(rows)]
;
;  [Mode]
;   Bit<7>-<5> --- NOT USED
;   Bit<4> ------- LCD SELECT ( 0:LCD1, 1:LCD2 )
;   Bit<3>-<0> --- Command for LCDC
;                  14:SET SPRITE POSITION
;
; [columns]
;   Bit<7> ----- Select block(0:left part,1:right part)
;   Bit<6>-<0>-- columns(0 - 95) + horizontal offset(cf. 6.1.)
;
; [rows]
;   Bit<7>-<2> - NOT USED
;   Bit<1>-<0> - rows(0,1,2,3)
;
; for example:
; set the position of the left(LCD1) sprite
;  PPO   &HDF         ;Select command register.
;  LD    $0,&H0E      ;Set "SET SPRITE POSITION","LCD1 select"
;  LD    $1,&H16      ;columns(8*2) + 6(horizontal offset)
;  LD    $2,&H01      ;rows=1
;  STLM  $0,3         ;set to lcdc(3 byte)
;  RTN
;
;Ŏw肷鐅ItZbgʒúAXvCg`ۂɐݒ肵lƓlłB
;\(columns)ʒúA8*2=16̔{ɏLItZbgʒulƂȂ܂B
;ȊO̒lݒ肵ꍇAXvCg\܂̂ŒӂĂB
;
;
; 6.5. [ SPRITE CONTROL SAMPLE(XvCgTv) ]
;
;  ;///(1) define the left(LCD1) sprite by graphic ///
;  PPO   &HDF          ;Select command register.
;  LD    $0,&H06       ;Set 'SPRITE DEFINITION BY GRAPHIC' & LCD1 select
;  LD    $1,6          ;horizontal offset = 3
;  STLM  $0,3          ;Set LCD command (3byte)
;  PRE   IX,SPRITE     ;
;  LDM   $0,(IX+$31),8 ;$0 - $7 <-- graphic data
;  PPO   &HDE          ;Select data ram
;  STLM  $0,8          ;set to lcdc(8 byte)
;
;  ;///(2) SET SPRITE DISPLAY WIDTH ///
;  PPO   &HDF          ;Select command register.
;  STL   &H08          ;Set SPRITE DISPLAY WIDTH=8
;
;  ;///(3) SET SPRITE POSITION ///
;  PPO   &HDF          ;Select command register.
;  LD    $0,&H0E       ;SET SPRITE POSITION
;  LD    $1,&H16       ;columns(8*2) + 6(horizontal offset=3)
;  LD    $2,&H01       ;rows=1
;  STLM  $0,3          ;Set LCD command (3byte)
;
;  ;///(4) SPRITE ON ///
;  PPO   &HDF          ; The sprite display is turned on. (XvCg\Onɂ)
;  STL   &HB9          ;Set "OR", sprite display on
;  RTN
;SPRITE:
;  DB &H3C,&H42,&H91,&H85 ;Graphic pattern data
;  DB &H85,&H91,&H42,&H3C ;
;
;ۂ̃XvCg\́AL(1)`(4)Ɏs邱Ƃŉ\łB
;
;
;////////////////////////////////////////
; 7. USER CHARACTER DEFINITION
;    [U[̒`
;////////////////////////////////////////
;
; [lcd command format]
;
;  [Mode],[chracter code]
;
;  [Mode]
;   Bit<7>-<4> --- NOT USED
;   Bit<3>-<0> --- Command for LCDC
;                  11:USER CHARACTER DEFINITION
;
; [character code]
;   Bit<7>-<2> --- NOT USED
;   Bit<1>-<0> --- code of the defined character
;		  00: Character &HFC
;		  01: Character &HFD
;		  10: Character &HFE
;		  11: Character &HFF
;
; for example:
; define the character &HFC and display (R[g&HFC֔Cӂ̃C[W`)
;  PPO   &HDF          ;Select command register.
;  LD    $0,&H0B       ;Set 'USER CHARACTER DEFINITION'
;  LD    $1,&H00       ;code of the defined character = &HFC
;  STLW  $0            ;Set LCD command (2byte)
;  PRE   IX,CHR_DATA   ;
;  LDM   $0,(IX+$31),8 ;$0 - $7 <-- graphic data
;  PPO   &HDE          ;Select data ram
;  STLM  $0,8          ;set to lcdc(8 byte)
;
;  PPO   &HDF          ;Select command register.
;  STL   &H08          ;Set CHARACTER WIDTH=8 ,TOP POSITION OF ROW=0(default)
;
;  PPO   &HDF          ;Select command register
;  LD    $0,&H83       ;Set 'DRAW CHARACTER','OVERWRITE', LCD1 select
;  LD    $1,32         ;Set LCD columns=(32/16)=2
;  LD    $2,0          ;Set LCD rows=0
;  STLM  $0,3          ;Set LCD command register & address(3byte)
;  LD    $3,&HCF       ;Set  character code = &HFC (&HFC --> &HCF)
;  PPO   &HDE          ;Select data ram
;  STL   $3            ;Display LCD( character data )
;  RTN
; CHR_DATA:
;  DB &H3C,&H42,&H91,&H85 ;Graphic pattern data
;  DB &H85,&H91,&H42,&H3C ;
;
;̃R}hɂALCDC̎LN^̂ALN^R[h&HFC`&HFF
;Cӂ̃OtBbNf[^ɂĒ`ėpł܂B
;(DEFCHR$CNȎg\ł)
;
;////////////////////////////////////////
; 8. SCROLL CONTROL
;    XN[
;////////////////////////////////////////
;
; [lcd command format]
;
;  [Mode]
;
; [Mode( 8bit )]
;   Bit<7>-<6> --- top position of row
;                  00 :row=0(default)
;                  01 :row=1
;                  10 :row=2
;                  11 :row=3
;   Bit<5>-<4> --- character(sprite display) width
;                  ( 00:8pixel ,01:7pixel ,10:6pixel(default) ,11:5pixel )
;   Bit<3>-<0> --- Command for LCDC
;                  8 :SET TOP POSITION OF ROW/SET CHARACTER(SPRITE) WIDTH
;
; for example:
;(1)Sample basic program for PB-1000/C.
; 5 'Push "SPACE"key to scroll.
; 10 CLS:DEFCHR$(240)="5400DF5228F7":EX=&H6B4A:ROW=0:CALL EX
; 20 PRINT"- Row = 0 -","+ Row = 1 +","* Row = 2 *","# Row = 3 #";
; 30 A$=INKEY$:IF A$<>" "THEN 30
; 40 POKE EX+4,&H28 OR(ROW*64):ROW=(ROW+1)AND3
; 50 CALL EX:GOTO30
;
; assembler part of sample program.
; define top position of row = 0 (\擪s0ɂ)
;  PPO   &HDF          ;Select command register.
;  STL   &H28          ;Set CHARACTER WIDTH=6(default) ,TOP POSITION OF ROW=0(default)
;  RTN
;
;(2) for AI-1000/PB-2000.
; 5 'Push "SPACE"key to scroll.
; 10 CLS:DEFCHR$(240)="5400DF522856":DEFCHR$(241)="6054F7000000"
; 15 EX=&H0A06:ROW=0:SYSTEM CALL EX
; 20 PRINT"- Row = 0 -","+ Row = 1 +","* Row = 2 *","# Row = 3 #";
; 30 A$=INKEY$:IF A$<>" "THEN 30
; 40 POKE EX+4,&H28 OR(ROW*64):ROW=(ROW+1)AND3
; 50 SYSTEM CALL EX:GOTO30
;
;(3) for Fx-870p/vx-4.
; 5 'Push "SPACE"key to scroll.
; 10 CLS:DEFCHR$(252)="5400DF522856":DEFCHR$(253)="6054F7000000"
; 15 EX=&H153C:ROW=0:MODE110(EX)
; 20 PRINT"- Row = 0 -","+ Row = 1 +","* Row = 2 *","# Row = 3 #";
; 30 A$=INKEY$:IF A$<>" "THEN 30
; 40 POKE EX+4,&H28 OR(ROW*64):ROW=(ROW+1)AND3
; 50 MODE110(EX):GOTO30
;
;(4) for vx-3.
; 5 'Push "SPACE"key to scroll.
; 10 CLS:DEFCHR$(252)="5400DF522856":DEFCHR$(253)="6054F7000000"
; 15 EX=&H6643:ROW=0:MODE110(EX)
; 20 PRINT"- Row = 0 -","+ Row = 1 +","* Row = 2 *","# Row = 3 #";
; 30 A$=INKEY$:IF A$<>" "THEN 30
; 40 POKE EX+4,&H28 OR(ROW*64):ROW=(ROW+1)AND3
; 50 MODE110(EX):GOTO30
;
; ̃R}hLCDCɑ΂row(s)ʒuݒAđ/邱ƂŁA
; LCDʑŜ㉺Ƀn[hEFAXN[邱Ƃł܂B
;
; ͔̑wi(LCD data ram)̕\؂ւ܂AXvCg\
; ύX܂BQ[ȂǂŔwiXN[ȂA@XvCg
; Œ\铙̎głłB
;
;------------------------------------------------------------
; MODEL NUMBER
;------------------------------------------------------------
PB1000: EQU     1               ;PB-1000/C
AI1000: EQU     2               ;AI-1000
FX870P: EQU     3               ;FX-870P/VX-4
VX3:    EQU     4               ;VX-3
;------------------------------------------------------------
MODEL:  EQU     PB1000          ;Select your machine.

;------------------------------------------------------------
; System work define.
;------------------------------------------------------------
#if (MODEL=PB1000)
LEDTP:  EQU     &H6201          ;LEDTP Address
BASE:   EQU     &H7000          ;Program start address
#endif
#if (MODEL=AI1000)
LEDTP:  EQU     &H0201          ;LEDTP Address
BASE:   EQU     &H1000          ;Program start address
#endif
#if (MODEL=FX870P)
LEDTP:  EQU     &H123C          ;LEDTP Address
BASE:   EQU     &H18D3          ;Program start address(calc)
#endif
#if (MODEL=VX3)
LEDTP:  EQU     &H6343          ;LEDTP Address
BASE:   EQU     &H68CC          ;Program start address(calc)
#endif

;------------------------------------------------------------
; DISPLCD
; compatible with DOTDS routine for PB-1000/C/AI-1000/FX-870P/VX-3
; INPUT : none
; USE REGISTER : $0 - $9 , IX
;------------------------------------------------------------
        START   DISPLCD
        ORG     BASE            ;
DISPLCD:
        PPO     &HDF            ;Set LCD (Select command register)
        LDW     $0,&H82         ;Set Bitimage mode(OR:&HA2,XOR:&H22)
        XR      $2,$2           ;Set LCD Address = 0,0
        STLM    $0,3            ;(write lcd register)

        PRE     IX,LEDTP        ;IX = LEDTP
        LD      $8,4            ;Set LCD line( = 4 )
LOOP:
        PPO     &HDF            ;Set LCD (Select command register)
        STL     &H82            ;select LCD1(left part of lcd)

        CAL     DISPL           ;LEDTP --> LCD( 96 byte )

        PPO     &HDF            ;Set LCD (Select command register)
        STL     &H92            ;select LCD2(right part of lcd)

        CAL     DISPL           ;LEDTP --> LCD( 96 byte )

        SB      $8,$30          ;finished ?
        JR      NZ,LOOP         ;

#if (MODEL=FX870P)
;        PST     UA,&H54        ;return to BASIC
#endif
        RTN                     ;

;----------------------------------
; INPUT : IZ -- LCDTP pointer
;----------------------------------
DISPL:
        PPO     &HDE            ;Set LCD (Select data ram)
        LD      $9,12           ;Set half line ( 12*8 = 96byte )
DLCDL:
        LDIM    $0,(IX+$31),8   ;Read bit data ( 8byte )
        STLM    $0,8            ;Display LCD ( 8byte )

        SB      $9,$30          ;finished ?
        JR      NZ,DLCDL        ;
        RTN
